import { useEffect } from "react";
import {
  useTagsAndCommitIds,
  useCSentryHost,
  useIocStatus
} from "../../api/SwaggerApi";

function onError(message) {
  console.warn(message);
}

export function IOCDetailFetcher({ ioc, index, setIocs }) {
  const [tagOrCommitId, getTagOrCommitId] = useTagsAndCommitIds(onError);
  const [host, getHost] = useCSentryHost(onError);
  const [iocStatus, getIocStatus] = useIocStatus();

  useEffect(() => {
    if (ioc.sourceVersion) {
      getTagOrCommitId(ioc.gitProjectId, ioc.sourceVersion);
    }
    if (ioc.activeDeployment) {
      getHost(ioc.activeDeployment.host.externalHostId);
    }
    getIocStatus(ioc.id);
  }, [ioc, getTagOrCommitId, getHost, getIocStatus]);

  useEffect(() => {
    setIocs((iocs) => {
      iocs[index].sourceVersionShort =
        tagOrCommitId?.length === 1
          ? tagOrCommitId[0].shortReference
          : iocs[index].sourceVersionShort;
      iocs[index].host = host ?? iocs[index].host;
      iocs[index].alert = iocStatus?.alertSeverity ?? iocs[index].alert;
      iocs[index].active = iocStatus?.active ?? iocs[index].active;
      return [...iocs];
    });
  }, [index, setIocs, tagOrCommitId, host, iocStatus]);

  return null;
}
