import { useCallback, useEffect, useMemo } from "react";
import { AdHocCommand, DeploymentStatus } from "../../../api/DataTypes";
import {
  useCommandById,
  useDeployment,
  useDeploymentJobById
} from "../../../api/SwaggerApi";
import { useSafePolling } from "../../../hooks/Polling";
import { STATUS_POLL_INTERVAL } from "./Notifications";
import {
  TYPE_COMMAND,
  TYPE_DEPLOYMENT,
  WATCHED_COMMAND_STORAGE_KEY,
  WATCHED_DEPLOYMENT_STORAGE_KEY
} from "./Storage";

export function DeploymentWatcher({
  deploymentId,
  addNotification,
  stopWatching
}) {
  const [deployment, getDeployment /* reset*/, , deploymentLoading] =
    useDeployment(deploymentId);
  const [deploymentJob, getDeploymentJobById /* reset*/, , jobLoading] =
    useDeploymentJobById();

  const getDeploymentJob = useCallback(() => {
    if (deployment?.awxJobId) {
      getDeploymentJobById(deployment?.awxJobId);
    }
  }, [deployment?.awxJobId, getDeploymentJobById]);

  useSafePolling(getDeployment, deploymentLoading, STATUS_POLL_INTERVAL);
  useSafePolling(getDeploymentJob, jobLoading, STATUS_POLL_INTERVAL);

  const deploymentStatus = useMemo(
    () => new DeploymentStatus(deployment, deploymentJob),
    [deployment, deploymentJob]
  );

  useEffect(() => {
    if (deploymentJob) {
      const message = `${deployment.namingName}: ${deploymentStatus.message()}`;
      addNotification(
        TYPE_DEPLOYMENT,
        deploymentId,
        deploymentStatus.status(),
        message
      );

      if (deploymentStatus.isFinished()) {
        stopWatching(deployment.id, WATCHED_DEPLOYMENT_STORAGE_KEY);
      }
    }
  }, [
    addNotification,
    deployment,
    deploymentId,
    deploymentJob,
    deploymentStatus,
    stopWatching
  ]);

  return null;
}

export function CommandWatcher({ commandId, addNotification, stopWatching }) {
  const [command, getCommand /* reset*/, , loading] = useCommandById();
  const poll = useCallback(
    () => getCommand(commandId),
    [commandId, getCommand]
  );
  useSafePolling(poll, loading, STATUS_POLL_INTERVAL);

  const commandStatus = useMemo(
    () => new AdHocCommand(command, null),
    [command]
  );

  useEffect(() => {
    if (command) {
      const message = `${command.iocName}: ${commandStatus.message()}`;
      addNotification(TYPE_COMMAND, commandId, command.status, message);

      if (commandStatus.isFinished()) {
        stopWatching(command.id, WATCHED_COMMAND_STORAGE_KEY);
      }
    }
  }, [addNotification, command, commandId, commandStatus, stopWatching]);

  return null;
}
